unit class_alternative;

interface

uses Graphics,
     Types,
     ContNrs,
     Dialogs,
     Math,
     class_element,
     class_instruction,
     class_subqueue,
     StrUtils,
     Classes,
     SysUtils,
     Utils;

type BAlternative =
                class(BElement)
                   function PrepareDraw(_canvas:TCanvas): TRect; override; // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect); override; // passes back the dimensions of the square
                   function getXml:UTF8string; override;
                   function getPas(_indent:word):TStringList; override;
                 public
                   qFalse,qTrue:BSubqueue;

                   constructor create; override;
                   destructor destroy; override;
                   procedure Free; override;

                   function SelectByCoord(_x,_y:word):boolean; override;
                   function SelectElementByCoord(_x,_y:word):BElement; override;
                   procedure setChildrenColor(_color:TColor); override;
                   function getACopyOf:BElement; override;
                 private
                   rFalse,rTrue:TRect;
                 protected  
                end;



implementation

constructor BAlternative.create;
begin
 inherited create;
 qFalse:=BSubqueue.create;
 qTrue:=BSubqueue.create;
 qFalse.parent:=self;
 qTrue.parent:=self;
end;

destructor BAlternative.destroy;
begin
 qFalse.free;
 qTrue.free;
 inherited destroy;
end;

procedure BAlternative.Free;
begin
 qFalse.free;
 qTrue.free;
 inherited free;
end;

function BAlternative.getACopyOf:BElement;
var ele:BElement;
    i:longint;
begin
  ele:=BAlternative.create;
  ele.comment.text:=self.comment.text;
  ele.text.text:=self.text.text;
  ele.color:=self.color;
  (ele as BAlternative).qTrue:=(self.qTrue.getACopyOf as BSubqueue);
  (ele as BAlternative).qFalse:=(self.qFalse.getACopyOf as BSubqueue);
  result:=ele;
end;

function BAlternative.PrepareDraw(_canvas:TCanvas): TRect;
var i:word;
begin
 rect.Top:=0;
 rect.Left:=0;

 rect.Right:=2*(E_PADDING div 2);
{ for i:=0 to text.count-1 do
  if (rect.Right<coloredTextOutWidth(text[i],vars,colors,_canvas)+2*(E_PADDING div 2))
   then rect.Right:=coloredTextOutWidth(text[i],vars,colors,_canvas)+2*(E_PADDING div 2);}

 for i:=0 to text.count-1 do
  if (rect.Right<2*_canvas.TextWidth(text[i])+2*(E_PADDING div 2))
   then rect.Right:=2*_canvas.TextWidth(text[i])+2*(E_PADDING div 2);

 rect.Bottom:=4*(E_PADDING div 2)+text.count*_canvas.TextHeight('O');
 //rect.Right:=2*_canvas.TextWidth(text)+2*(E_PADDING div 2);
 //rect.Bottom:=_canvas.TextHeight(text)+4*(E_PADDING div 2);

 rFalse:=qFalse.PrepareDraw(_canvas);
 rTrue:=qTrue.PrepareDraw(_canvas);

 //rect.Right:=max(rect.Right,max(2*rTrue.Right,2*rFalse.Right));
 rect.Right:=max(rect.Right,rTrue.Right+rFalse.Right);
 rect.Bottom:=rect.bottom+max(rFalse.bottom,rTrue.Bottom);

 result:=rect;
end;

procedure BAlternative.Draw(_canvas:TCanvas; _top_left:TRect);
var myrect:TRect;
    drawColor,colCol:TColor;
    a,b,c,d,x,y:integer;
    wmax,i,p,w:word;
    fn:UTF8string;
begin
 drawColor:=clWhite;
 drawColor:=color;
 if (selected=true) then drawColor:=E_DRAWCOL;

 _canvas.Brush.Color:=clBlack;
 _canvas.FillRect(_top_left);

 rect:=_top_left;
 _canvas.brush.Color:=drawColor;
 myrect:=_top_left;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right-1;
 _canvas.FillRect(myrect);


 rect:=_top_left;
 myrect:=_top_left;
 myrect.Bottom:=_top_left.top+text.count*_canvas.TextHeight(text[0])+4*(E_PADDING div 2);
 //myrect.Bottom:=_top_left.top+_canvas.TextHeight(text)+4*(E_PADDING div 2);
 _canvas.brush.Color:=drawColor;
// _canvas.TextOut(myrect.left+((myrect.right-myrect.left) div 2)-(_canvas.TextWidth(text) div 2),
//                 myrect.Top+(E_PADDING div 2),text );
 y:=myrect.top+E_PADDING;
 a:=myrect.left+((myrect.right-myrect.left) div 2);
 b:=myrect.top;
 c:=myrect.left+rTrue.Right-1;
 d:=myrect.Bottom-1;
 x:=trunc(((y-b)*(c-a)+a*(d-b))/(d-b));

 wmax:=0;

 for i:=0 to text.count-1 do
  if (wmax<coloredTextOutWidth(text[i],vars,colors,_canvas))
   then wmax:=coloredTextOutWidth(text[i],vars,colors,_canvas);

{ for i:=0 to text.count-1 do
  if (wmax<_canvas.TextWidth(text[i]))
   then wmax:=_canvas.TextWidth(text[i]);}


 for i:=0 to text.count-1 do
 begin
   coloredTextOut(text[i],vars,colors,_canvas,x-(coloredTextOutWidth(text[i],vars,colors,_canvas) div 2),
                 myrect.Top+(E_PADDING div 3)+i*_canvas.TextHeight('O'));
{ _canvas.TextOut(x-(_canvas.TextWidth(text[i]) div 2),
                 myrect.Top+(E_PADDING div 2)+i*_canvas.TextHeight('O'),text[i] );}
 end;
// _canvas.TextOut(x-(_canvas.TextWidth(text) div 2),
//                 myrect.Top+(E_PADDING div 2),text );

// _canvas.TextOut(myrect.left+rTrue.Right-(_canvas.TextWidth(text) div 2),
//                 myrect.Top+(E_PADDING div 2),text );
 _canvas.TextOut(myrect.Left+(E_PADDING div 4),
                 myrect.bottom-(E_PADDING div 4)-_canvas.TextHeight(NSDUtils.txtTrue),NSDUtils.txtTrue);
 _canvas.TextOut(myrect.right-(E_PADDING div 4)-_canvas.TextWidth(NSDUtils.txtFalse),
                 myrect.bottom-(E_PADDING div 4)-_canvas.TextHeight(NSDUtils.txtFalse),NSDUtils.txtFalse);
// _canvas.Brush.Color:=clBlack;
// _canvas.FrameRect(myrect);
 if((trim(comment.Text)<>'')and(NSDUtils.flagDrawComments=true)) then
 begin
  colCol:=_canvas.Pen.Color;
  _canvas.Pen.Color:=commentColor;
  _canvas.Pen.Width:=2;
  _canvas.MoveTo(myrect.Left+3,myrect.Top+3);
  _canvas.LineTo(myrect.Left+3,myrect.Bottom-3);
  _canvas.Pen.Width:=1;
  _canvas.Pen.Color:=colCol;
 end;
 _canvas.MoveTo(myrect.left,myrect.Top);
// _canvas.LineTo(myrect.left+((myrect.right-myrect.left)div 2)-1,myrect.Bottom-1);
 _canvas.LineTo(myrect.left+rTrue.Right-1,myrect.Bottom-1);
 _canvas.LineTo(myrect.right,myrect.Top);

 myrect:=_top_left;
 myrect.top:=_top_left.top+_canvas.TextHeight('O')*text.count+4*(E_PADDING div 2)-1;
// myrect.Right:=myrect.Left+((myrect.Right-myrect.Left) div 2);
 myrect.Right:=myrect.Left+rTrue.Right;
 qTrue.Draw(_canvas,myrect);
 myrect.left:=myrect.right-1;
 myrect.right:=_top_left.Right;
 qFalse.Draw(_canvas,myrect);

end;

function BAlternative.SelectByCoord(_x,_y:word):boolean;
var seleT,seleF:boolean;
begin
 inherited SelectByCoord(_x,_y);
 seleT:=qTrue.SelectByCoord(_x,_y);
 seleF:=qFalse.SelectByCoord(_x,_y);
 if ((seleT=true) or (seleF=true)) then selected:=false;
 result:=(selected or seleT or seleF);
end;

function BAlternative.SelectElementByCoord(_x,_y:word):BElement;
var seleT,seleF:BElement;
begin
 result:=inherited SelectElementByCoord(_x,_y);
 seleT:=qTrue.SelectElementByCoord(_x,_y);
 seleF:=qFalse.SelectElementByCoord(_x,_y);
 if(seleT<>nil) then result:=seleT
 else if (selef<>nil) then result:=seleF;
end;

function BAlternative.getXml:UTF8string;
begin
 result:='<alternative text="'+xmlTransform(text.commatext)+'" comment="'+xmlTransform(comment.commatext)+'" color="'+Rev(Inttohex(Color,6))+'">'+#13+
         ' <qTrue>'+#13+
         qTrue.getXml+#13+
         ' </qTrue>'+#13+
         ' <qFalse>'+#13+
         qFalse.getXml+#13+
         ' </qFalse>'+#13+
         '</alternative>';
end;

function BAlternative.getPas(_indent:word):TStringList;
var s:TStringList;
    ou:UTF8string;
    i:word;
begin
 ou:='';
 if(text.count>0) then for i:=0 to text.count-1 do ou:=ou+trim(text[i])+' ';
 s:=TStringList.create;
 if(comment.Count>0) then
 begin
   for i:=0 to comment.Count-1 do
   begin
     s.add(makeIndent(_indent)+'// '+comment[i]+';');
   end;
 end;
 s.add(makeIndent(_indent)+'if '+ou+' then');
 s.add(makeIndent(_indent)+'begin');
 s.AddStrings(qTrue.getPas(_indent+E_INDENT));
 if (qFalse.children.Count>0) then
 begin
  s.add(makeIndent(_indent)+'end');
  s.add(makeIndent(_indent)+'else');
  s.add(makeIndent(_indent)+'begin');
  s.AddStrings(qFalse.getPas(_indent+E_INDENT));
 end; 
 s.add(makeIndent(_indent)+'end;');
 result:=s;
end;

procedure BAlternative.setChildrenColor(_color:TColor);
begin
  color:=_color;
  qTrue.setChildrenColor(_color);
  qFalse.setChildrenColor(_color);
end;




end.
