unit class_case;

interface

uses Graphics,
     Types,
     ContNrs,
     Dialogs,
     Math,
     class_element,
     class_instruction,
     class_subqueue,
     StrUtils,
     Classes,
     SysUtils,
     Utils;

type BCase =
                class(BElement)
                   function PrepareDraw(_canvas:TCanvas): TRect; override; // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect); override; // passes back the dimensions of the square
                   function getXml:UTF8string; override;
                   function getPas(_indent:word):TStringList; override;

                 public
                   qs:TObjectList;

                   constructor create; override;
                   destructor destroy; override;
                   procedure Free; override;

                   function SelectByCoord(_x,_y:word):boolean; override;
                   function SelectElementByCoord(_x,_y:word):BElement; override;
                   procedure setChildrenColor(_color:TColor); override;
                   function getACopyOf:BElement; override;

                   procedure setTextLines(_aText:UTF8string);

                 private
                   fullwidth,maxheight:word;
                   procedure setText(_aText:TStringList);
                 protected
                 published
                end;



implementation

uses class_root;

procedure BCase.setText(_aText:TStringList);
begin
  f_text.text:=_aText.text;
  if(text.Count<=1) then
  begin
   while(qs.count<>0) do qs.delete(0);
  end
  else
  begin
   while((text.count)-1<qs.count) do qs.delete(qs.Count-1);
   while((text.count)-1>qs.count) do qs.add(BSubqueue.create);
  end;
end;

procedure BCase.setTextLines(_aText:UTF8string);
var str:TStringlist;
begin
  str:=TStringlist.Create;
  str.text:=_aText;
  setText(str);
  str.free;
end;

constructor BCase.create;
begin
 inherited create;
 qs:=TObjectList.create(false);
end;

destructor BCase.destroy;
begin
 while(qs.count<>0) do qs.delete(0);
 qs.free;
 inherited destroy;
end;

procedure BCase.Free;
begin
 while(qs.count<>0) do qs.delete(0);
 qs.free;
 inherited free;
end;

function BCase.getACopyOf:BElement;
var ele:BCase;   // 1.25
    i:longint;
begin
  ele:=BCase.create;
  ele.comment.text:=self.comment.text;
  ele.text.text:=self.text.text;
  ele.color:=self.color;                                                        // 1.25
  if (self.qs.count<>0) then                                                    // 1.25
    for i:=0 to self.qs.count-1 do                                              // 1.25
      ele.qs.Add((self.qs.Items[i] as BSubqueue).getACopyOf);                   // 1.25
  result:=ele;
end;

function BCase.PrepareDraw(_canvas:TCanvas): TRect;
var i:word;
    rtt:TRect;
begin
 rect.Top:=0;
 rect.Left:=0;

 rect.Right:=2*(E_PADDING div 2);

 for i:=0 to text.count-1 do
  if (rect.Right<2*_canvas.TextWidth(text[i])+2*(E_PADDING div 2))
   then rect.Right:=2*_canvas.TextWidth(text[i])+2*(E_PADDING div 2);

 rect.Bottom:=4*(E_PADDING div 2)+2*_canvas.TextHeight('O');

 fullwidth:=0;
 maxheight:=0;
 if (qs.count<>0) then
   for i:=0 to qs.count-1 do
   begin
     rtt:=(qs[i] as BSubqueue).PrepareDraw(_canvas);
     fullwidth:=fullwidth+max(rtt.Right,_canvas.TextWidth(text[i+1])+(E_PADDING div 2));
     if(maxheight<rtt.Bottom) then maxheight:=rtt.Bottom;
   end;

 rect.Right:=max(rect.Right,fullwidth);
 rect.Bottom:=rect.bottom+maxheight;

 result:=rect;
end;

procedure BCase.Draw(_canvas:TCanvas; _top_left:TRect);
var myrect:TRect;
    drawColor,colCol:TColor;
    a,b,c,d,x,y:integer;
    wmax,i,p,w:word;
    fn:UTF8string;
    linewidth:word;
    rtt:TRect;
    ax,ay,bx,by,mx,my,sx,sy:integer;
begin
 drawColor:=clWhite;
 drawColor:=color;
 if (selected=true) then drawColor:=E_DRAWCOL;
 
 _canvas.brush.Color:=clBlack;
 _canvas.FillRect(_top_left);


 // big border and background
 rect:=_top_left;
 _canvas.brush.Color:=drawColor;
 myrect:=_top_left;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right-1;
 _canvas.FillRect(myrect);

 rect:=_top_left;
 myrect:=_top_left;
 myrect.Bottom:=_top_left.top+2*_canvas.TextHeight(text[0])+4*(E_PADDING div 2);
(* _canvas.FillRect(myrect);
 _canvas.brush.Color:=drawColor;*)

 y:=myrect.top+E_PADDING;
 a:=myrect.left+((myrect.right-myrect.left) div 2);
 b:=myrect.top;
 c:=myrect.left+fullwidth-1;
 d:=myrect.Bottom-1;
 x:=trunc(((y-b)*(c-a)+a*(d-b))/(d-b));
{
 wmax:=0;
 for i:=0 to text.count-1 do
  if (wmax<coloredTextOutWidth(text[i],vars,colors,_canvas))
   then wmax:=coloredTextOutWidth(text[i],vars,colors,_canvas);

}

 for i:=0 to 0 do
 begin
   coloredTextOut(text[i],vars,colors,_canvas,x-(coloredTextOutWidth(text[i],vars,colors,_canvas) div 2),
                 myrect.Top+(E_PADDING div 3)+i*_canvas.TextHeight('O'));
 end;
{
 _canvas.TextOut(myrect.Left+(E_PADDING div 4),
                 myrect.bottom-(E_PADDING div 4)-_canvas.TextHeight(getTrue),getTrue);
 _canvas.TextOut(myrect.right-(E_PADDING div 4)-_canvas.TextWidth(getFalse),
                 myrect.bottom-(E_PADDING div 4)-_canvas.TextHeight(getFalse),getFalse);
}
 // draw comments line
// _canvas.Brush.Color:=clBlack;
// _canvas.FrameRect(myrect);
 if((trim(comment.Text)<>'')and(NSDUtils.flagDrawComments=true)) then
 begin
  colCol:=_canvas.Pen.Color;
  _canvas.Pen.Color:=commentColor;
  _canvas.Pen.Width:=2;
  _canvas.MoveTo(myrect.Left+3,myrect.Top+3);
  _canvas.LineTo(myrect.Left+3,myrect.Bottom-3);
  _canvas.Pen.Width:=1;
  _canvas.Pen.Color:=colCol;
 end;

 // draw lines
 linewidth:=0;                                                                  // 1.22
 for i:=0 to qs.count-2 do                                                      // 1.22
 begin                                                                          // 1.22
   rtt:=(qs[i] as BSubqueue).PrepareDraw(_canvas);                              // 1.22
   linewidth:=linewidth+max(rtt.Right,
                            _canvas.TextWidth(text[i+1])+(E_PADDING div 2));    // 1.22
 end;                                                                           // 1.22

 _canvas.MoveTo(myrect.left,myrect.Top);                                                          // = A
 ax:=myrect.left;                                                               // 1.25
 ay:=myrect.Top;                                                                // 1.25
 _canvas.LineTo(myrect.left+linewidth,myrect.Bottom-1-_canvas.TextHeight('O')-(E_PADDING div 2)); // = B
 bx:=myrect.left+linewidth;                                                     // 1.25
 by:=myrect.Bottom-1-_canvas.TextHeight('O')-(E_PADDING div 2);                 // 1.25
 _canvas.LineTo(myrect.left+linewidth,myrect.Bottom-1);
 _canvas.LineTo(myrect.left+linewidth,myrect.Bottom-1-_canvas.TextHeight('O')-(E_PADDING div 2));
 _canvas.LineTo(myrect.right,myrect.Top);



 // draw children
 myrect:=_top_left;
 myrect.top:=_top_left.top+_canvas.TextHeight('O')*2+4*(E_PADDING div 2)-1;
 if (qs.count<>0) then
   for i:=0 to qs.count-1 do
   begin
     rtt:=(qs[i] as BSubqueue).PrepareDraw(_canvas);
     if(i=qs.count-1) then myrect.Right:=_top_left.right                // 1.22
     else if((i<>qs.count-1)or(not(self.parent.parent is BRoot))) then myrect.Right:=myrect.Left+max(rtt.Right,_canvas.TextWidth(text[i+1])+(E_PADDING div 2))+1
          else myrect.Right:=myrect.Left+max(rtt.Right,_canvas.TextWidth(text[i+1])+(E_PADDING div 2));
     (qs[i] as BSubqueue).Draw(_canvas,myrect);
//     _canvas.Brush.Style:=bsClear;
     _canvas.TextOut(myrect.right+((myrect.left-myrect.right) div 2)-(_canvas.TextWidth(text[i+1]) div 2),
                     myrect.top-(E_PADDING div 4)-_canvas.TextHeight(text[i+1]),text[i+1]);
     if(i<>qs.count-1) then                                             //1.22
     begin                                                              //1.22
       _canvas.MoveTo(myrect.right-1,myrect.Top);                               
       //_canvas.LineTo(myrect.right-1,myrect.Top-_canvas.TextHeight('O'));     // 1.25
       mx:=myrect.right-1;                                                      // 1.25
       my:=myrect.Top-_canvas.TextHeight('O');                                  // 1.25
       sx:=mx;                                                                  // 1.25
       sy:=round((sx*(by-ay)-ax*by+ay*bx)/(bx-ax));                             // 1.25
       _canvas.LineTo(sx,sy);                                                   // 1.25
     end;                                                               //1.22
     myrect.left:=myrect.right-1;
   end;

end;

function BCase.SelectByCoord(_x,_y:word):boolean;
var sele:boolean;
    i:word;
begin
 inherited SelectByCoord(_x,_y);
 sele:=false;
 if (qs.count<>0) then
   for i:=0 to qs.count-1 do
     sele:=sele or (qs[i] as BSubqueue).SelectByCoord(_x,_y);
 if (sele=true) then selected:=false;
 result:=(selected or sele);
end;

function BCase.SelectElementByCoord(_x,_y:word):BElement;
var sele,pre:BElement;
    i:word;
begin
 result:=inherited SelectElementByCoord(_x,_y);
 sele:=nil;
 if (qs.count<>0) then
   for i:=0 to qs.count-1 do
   begin
     pre:=(qs[i] as BSubqueue).SelectElementByCoord(_x,_y);
     if(pre<>nil) then sele:=pre;
   end;
 if(sele<>nil) then result:=sele;
end;

function BCase.getXml:UTF8string;
var i:word;
begin
 result:='<case text="'+xmlTransform(text.commatext)+'" comment="'+xmlTransform(comment.commatext)+'" color="'+Rev(Inttohex(Color,6))+'">'+#13;
 if (qs.count<>0) then
   for i:=0 to qs.count-1 do
   begin
     result:=result+
             '<qCase>'+#13+
             (qs[i] as BSubqueue).getXml+#13+
             '</qCase>'+#13;
   end;
  result:=result+'</case>';
end;

function BCase.getPas(_indent:word):TStringList;
var s:TStringList;
    ou:UTF8string;
    i:word;
begin
 ou:='';
 if(text.count>0) then for i:=0 to 0 do ou:=ou+trim(text[i])+' ';
 s:=TStringList.create;
 if(comment.Count>0) then
 begin
   for i:=0 to comment.Count-1 do
   begin
     s.add(makeIndent(_indent)+'// '+comment[i]+';');
   end;
 end;
 s.add(makeIndent(_indent)+'case '+ou+' of');
 if (qs.count<>0) then
 begin
   for i:=0 to qs.count-2 do
   begin
     s.add(makeIndent(_indent+1*E_INDENT)+text[i+1]+':');
     s.add(makeIndent(_indent+2*E_INDENT)+'begin');
     s.AddStrings((qs[i] as BSubqueue).getPas(_indent+3*E_INDENT));
     s.add(makeIndent(_indent+2*E_INDENT)+'end;');
   end;
 end;
 s.add(makeIndent(_indent)+'else');
 s.AddStrings((qs[qs.count-1] as BSubqueue).getPas(_indent+1*E_INDENT));
 s.add(makeIndent(_indent)+'end;');
 result:=s;
end;

procedure BCase.setChildrenColor(_color:TColor);
var i:word;
begin
  color:=_color;
 if (qs.count<>0) then
   for i:=0 to qs.count-1 do (qs[i] as BSubqueue).setChildrenColor(_color);
end;




end.
