unit class_element;

interface

uses Graphics,
     Types,
     ContNrs,
     Math,
     Dialogs,
     SysUtils,
     StrUtils,
     Classes;

const E_PADDING = 20;
      E_INDENT  = 2;

type BElement = class
                   f_text,f_comment:TStringlist;
                   function PrepareDraw(_canvas:TCanvas): TRect; virtual; abstract;  // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect); virtual; abstract;  // passes back the dimensions of the square
                 private
                 public
                   parent:BElement;
                   selected:boolean;
                   color:TColor;

                   constructor create; virtual;
                   destructor destroy; virtual;
                   procedure Free; virtual;

                   function SelectByCoord(_x,_y:word):boolean; virtual;
                   function SelectElementByCoord(_x,_y:word):BElement; virtual;

                   function getACopyOf:BElement; virtual; abstract;

                   function getXml:UTF8string; virtual; abstract;
                   function getPas(_indent:word):TStringlist; virtual; abstract;
                   procedure setChildrenColor(_color:TColor); virtual; abstract;
                 protected
                   rect:TRect;
                 published
                   property text: TStringlist read f_text write f_text;
                   property comment: TStringlist read f_comment write f_comment;
                end;


function xmlTransform(_str:UTF8string): UTF8string;
function xmlUnTransform(_str:UTF8string): UTF8string;
function makeIndent(_ident:word): UTF8string;
Function Rev(_s:UTF8string): UTF8string;

var E_DRAWCOL:Longword;

implementation

{
 Text_Color_Dialog.Color:=StrtoInt('$'+Rev(Box_Color.text));
 Box_Color.text:=Rev(Inttohex(Text_Color_Dialog.Color,6));
}

Function Rev(_s:UTF8string): UTF8string;
var s:UTF8string;
begin
 s:=_s;
 s[1]:=_s[5];
 s[2]:=_s[6];
 s[5]:=_s[1];
 s[6]:=_s[2];
 rev:=copy(s,1,6);
end;

constructor BElement.create;
begin
 parent:=nil;
 selected:=false;
 f_text:=TStringlist.create;
 f_comment:=TStringlist.create;
 color:=clWhite;
end;

destructor BElement.destroy;
begin
 parent:=nil;
 f_text.free;
 f_comment.free;
 inherited destroy;
end;

procedure BElement.Free;
begin
 f_text.free;
 f_comment.free;
 inherited free;
end;

function BElement.SelectByCoord(_x,_y:word):boolean;
begin
 if ((rect.Left<_x)and(_x<rect.Right)and
     (rect.Top<_y)and(_y<rect.Bottom)) then selected:=true
 else selected:=false;
 result:=selected;
end;

function BElement.SelectElementByCoord(_x,_y:word):BElement;
begin
 if ((rect.Left<_x)and(_x<rect.Right)and
     (rect.Top<_y)and(_y<rect.Bottom)) then
 begin
  result:=self;
 end
 else
 begin
  result:=nil;
  selected:=false;
 end;
end;

function xmlTransform(_str:UTF8string): UTF8string;
begin
 _str:=AnsiReplaceText(_str,'&','&#38;');
 _str:=AnsiReplaceText(_str,'"','&#34;');
 _str:=AnsiReplaceText(_str,'<','&#60;');
 _str:=AnsiReplaceText(_str,'>','&#62;');
 result:=_str;
end;


function xmlUnTransform(_str:UTF8string): UTF8string;
begin
 {$IFDEF DARWIN}
 _str:=ansitoutf8(_str);
 {$ENDIF}
 _str:=AnsiReplaceText(_str,'&#38;','&');
 _str:=AnsiReplaceText(_str,'&#34;','"');
 _str:=AnsiReplaceText(_str,'&#60;','<');
 _str:=AnsiReplaceText(_str,'&#62;','>');
 result:=_str;
end;


function makeIndent(_ident:word): UTF8string;
var i:word;
    s:UTF8string;
begin
 s:='';
 for i:=1 to _ident do s:=s+' ';
 result:=s;
end;

begin
 E_DRAWCOL:=clYellow;
end.
