unit class_for;

interface

uses Graphics,
     Types,
     ContNrs,
     Dialogs,
     Math,
     class_element,
     class_instruction,
     class_subqueue,
     StrUtils,
     Classes,
     SysUtils,
     Utils;

type BFor =
                class(BElement)
                   function PrepareDraw(_canvas:TCanvas): TRect; override; // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect); override; // passes back the dimensions of the square
                   function getXml:UTF8string; override;
                   function getPas(_indent:word):TStringList; override;
                 public
                   q:BSubqueue;

                   constructor create; override;
                   destructor destroy; override;
                   procedure Free; override;

                   function SelectByCoord(_x,_y:word):boolean; override;
                   function SelectElementByCoord(_x,_y:word):BElement; override;
                   procedure setChildrenColor(_color:TColor); override;
                   function getACopyOf:BElement; override;
                 private
                   r:TRect;
                 protected
                end;


implementation


constructor BFor.create;
begin
 inherited create;
 q:=BSubqueue.create;
 q.parent:=self;
end;

destructor BFor.destroy;
begin
 q.free;
 inherited destroy;
end;

procedure BFor.Free;
begin
 q.free;
 inherited free;
end;

function BFor.getACopyOf:BElement;
var ele:BElement;
    i:longint;
begin
  ele:=BFor.create;
  ele.comment.text:=self.comment.text;
  ele.text.text:=self.text.text;
  ele.color:=self.color;
  (ele as BFor).q:=(self.q.getACopyOf as BSubqueue);
  result:=ele;
end;

function BFor.PrepareDraw(_canvas:TCanvas): TRect;
var i:word;
begin
 rect.Top:=0;
 rect.Left:=0;
 rect.Right:=2*(E_PADDING div 2);
 for i:=0 to text.count-1 do
  if (rect.Right<coloredTextOutWidth(text[i],vars,colors,_canvas)+2*(E_PADDING div 2))
   then rect.Right:=coloredTextOutWidth(text[i],vars,colors,_canvas)+2*(E_PADDING div 2);

{ for i:=0 to text.count-1 do
  if (rect.Right<_canvas.TextWidth(text[i])+2*(E_PADDING div 2))
   then rect.Right:=_canvas.TextWidth(text[i])+2*(E_PADDING div 2);}
   
 rect.Bottom:=2*(E_PADDING div 2)+text.count*_canvas.TextHeight('O');
// rect.Right:=_canvas.TextWidth(text)+2*(E_PADDING div 2);
// rect.Bottom:=_canvas.TextHeight(text)+2*(E_PADDING div 2);

 r:=q.PrepareDraw(_canvas);

 rect.Right:=max(rect.Right,r.Right+E_PADDING);
 rect.Bottom:=rect.bottom+r.bottom+E_PADDING;

 result:=rect;
end;

procedure BFor.Draw(_canvas:TCanvas; _top_left:TRect);
var myrect:TRect;
    drawColor,colcol:TColor;
    i,p,w:word;
    fn:UTF8string;
begin
 drawColor:=clWhite;
 drawColor:=color;
 if (selected=true) then drawColor:=E_DRAWCOL;

 rect:=_top_left;
 _canvas.Brush.Color:=clBlack;
 _canvas.FrameRect(_top_left);

 _canvas.Brush.Color:=clBlack;
 myrect:=_top_left;
 myrect.Bottom:=_top_left.top+_canvas.TextHeight('O')*text.count+2*(E_PADDING div 2);
 _canvas.FrameRect(myrect);

 _canvas.Brush.Color:=clBlack;
 myrect.Bottom:=_top_left.Bottom;
 myrect.Top:=myrect.Bottom-E_PADDING;
 _canvas.FrameRect(myrect);

 _canvas.Brush.Color:=clBlack;
 myrect:=_top_left;
 myrect.Right:=myrect.Left+E_PADDING;
 _canvas.FrameRect(myrect);
 _canvas.Brush.Color:=drawColor;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right-1;
 _canvas.FillRect(myrect);

 myrect:=_top_left;
 myrect.Bottom:=_top_left.top+_canvas.TextHeight('0')*text.count+2*(E_PADDING div 2);
 _canvas.Brush.Color:=drawColor;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right-1;
 _canvas.FillRect(myrect);

 myrect.Bottom:=_top_left.Bottom;
 myrect.Top:=myrect.Bottom-E_PADDING;
 _canvas.Brush.Color:=drawColor;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right;
 _canvas.FillRect(myrect);
 if((trim(comment.Text)<>'')and(NSDUtils.flagDrawComments=true)) then
 begin
  colCol:=_canvas.Pen.Color;
  _canvas.Pen.Color:=commentColor;
  _canvas.Pen.Width:=2;
  _canvas.MoveTo(myrect.Left+1,_top_left.Top+2);
  _canvas.LineTo(myrect.Left+1,_top_left.Bottom-3);
  _canvas.Pen.Width:=1;
  _canvas.Pen.Color:=colCol;
 end;

 myrect:=_top_left;
 _canvas.brush.Color:=drawColor;

 for i:=0 to text.Count-1 do
 begin
   text[i]:=AnsiReplaceStr(text[i],'<--','<-');
//   text[i]:=AnsiReplaceStr(text[i],'<=','<-');
//   text[i]:=AnsiReplaceStr(text[i],'<==','<-');
   coloredTextOut(text[i],vars,colors,_canvas,_top_left.Left+(E_PADDING div 2),
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]));

{   p:=pos('<-',text[i]);
   if(p=0)then
   _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),text[i])
   else
   begin
    _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
                    _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),copy(text[i],1,p-1));
    w:=_canvas.TextWidth(copy(text[i],1,p-1));
    fn:=_canvas.Font.Name;
    _canvas.Font.Name:='Symbol';
    _canvas.TextOut(_top_left.Left+(E_PADDING div 2)+w,
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i])-2,'');
    _canvas.Font.Name:=fn;
    _canvas.TextOut(_top_left.Left+(E_PADDING div 2)+_canvas.TextWidth(copy(text[i],1,p+1)),
                    _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),copy(text[i],p+2,length(text[i])));
   end;}
 end;
{   _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),text[i]);}
// _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
//                 _top_left.Top+(E_PADDING div 2),text );

 myrect:=_top_left;
 myrect.Left:=myrect.Left+E_PADDING-1;
 myrect.Top:=_top_left.top+_canvas.TextHeight('O')*text.count+2*(E_PADDING div 2)-1;
 myrect.Bottom:=myrect.Bottom-E_PADDING+1;
 q.draw(_canvas,myrect)
end;

function BFor.SelectByCoord(_x,_y:word):boolean;
var sel:boolean;
begin
 inherited SelectByCoord(_x,_y);
 sel:=q.SelectByCoord(_x,_y);
 if(sel=true) then selected:=false;
 result:=(selected or sel);
end;

function BFor.SelectElementByCoord(_x,_y:word):BElement;
var sel:BElement;
begin
 result:=inherited SelectElementByCoord(_x,_y);
 sel:=q.SelectElementByCoord(_x,_y);
 if(sel<>nil) then result:=sel;
end;

function BFor.getXml:UTF8string;
begin
 result:='<for text="'+xmlTransform(text.commatext)+'" comment="'+xmlTransform(comment.commatext)+'" color="'+Rev(Inttohex(Color,6))+'">'+#13+
         ' <qFor>'+#13+
         q.getXml+#13+
         ' </qFor>'+#13+
         '</for>';
end;

function BFor.getPas(_indent:word):TStringList;
var s:TStringList;
    ou:UTF8string;
    i:word;
begin
 ou:='';
 if(text.count>0) then for i:=0 to text.count-1 do ou:=ou+trim(text[i]);
 s:=TStringList.create;
 if(comment.Count>0) then
 begin
   for i:=0 to comment.Count-1 do
   begin
     s.add(makeIndent(_indent)+'// '+comment[i]+';');
   end;
 end;
 s.add(makeIndent(_indent)+'for '+ou+' do');
 s.add(makeIndent(_indent)+'begin');
 s.AddStrings(q.getPas(_indent+E_INDENT));
 s.add(makeIndent(_indent)+'end;');
 result:=s;
end;

procedure Bfor.setChildrenColor(_color:TColor);
begin
  color:=_Color;
  q.setChildrenColor(_color);
end;

end.
