unit class_instruction;

interface

uses Graphics,
     Types,
     ContNrs,
     Dialogs,
     class_element,
     StrUtils,
     RegExpr,
     SysUtils,
     Classes,
     Utils;

type BInstruction =
                class(BElement)
                   function PrepareDraw(_canvas:TCanvas): TRect; override; // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect); override; // passes back the dimensions of the square
                   function getXml:UTF8string; override;
                   function getPas(_indent:word):TStringList; override;
                   procedure setChildrenColor(_color:TColor); override;
                   function getACopyOf:BElement; override;
                 public
                 private
                 protected
                end;

implementation

function BInstruction.PrepareDraw(_canvas:TCanvas): TRect;
var i:word;
begin
 rect.Top:=0;
 rect.Left:=0;
 rect.Right:=2*(E_PADDING div 2);

 for i:=0 to text.count-1 do
  if (rect.Right<coloredTextOutWidth(text[i],vars,colors,_canvas)+2*(E_PADDING div 2))
   then rect.Right:=coloredTextOutWidth(text[i],vars,colors,_canvas)+2*(E_PADDING div 2);

{ for i:=0 to text.count-1 do
  if (rect.Right<_canvas.TextWidth(text[i])+2*(E_PADDING div 2))
   then rect.Right:=_canvas.TextWidth(text[i])+2*(E_PADDING div 2);}
 //rect.Right:=_canvas.TextWidth(text)+2*(E_PADDING div 2);
 rect.Bottom:=2*(E_PADDING div 2)+text.count*_canvas.TextHeight('O');
 //rect.Bottom:=_canvas.TextHeight(text)+2*(E_PADDING div 2);
 result:=rect;
end;

function BInstruction.getACopyOf:BElement; 
var ele:BElement;
begin
  ele:=BInstruction.create;
  ele.comment.text:=self.comment.text;
  ele.text.Text:=self.text.Text;
  ele.color:=self.color;
  result:=ele;
end;

procedure BInstruction.Draw(_canvas:TCanvas; _top_left:TRect);
var myrect:TRect;
    drawColor,colCol:TColor;
    i,p,w:word;
    fn:UTF8string;
begin
 drawColor:=clWhite;
 drawColor:=color;
 if (selected=true) then drawColor:=E_DRAWCOL;

 _canvas.Brush.Color:=clBlack;
 _canvas.FillRect(_top_left);

 rect:=_top_left;
 _canvas.brush.Color:=drawColor;
 myrect:=_top_left;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right-1;
 _canvas.FillRect(myrect);
 if((trim(comment.Text)<>'')and(NSDUtils.flagDrawComments=true)) then
 begin
  colCol:=_canvas.Pen.Color;
  _canvas.Pen.Color:=commentColor;
  _canvas.Pen.Width:=2;
  _canvas.MoveTo(myrect.Left+2,myrect.Top+1);
  _canvas.LineTo(myrect.Left+2,myrect.Bottom-2);
  _canvas.Pen.Width:=1;
  _canvas.Pen.Color:=colCol;
 end;

 for i:=0 to text.Count-1 do
 begin
   text[i]:=AnsiReplaceStr(text[i],'<--','<-');
//   text[i]:=AnsiReplaceStr(text[i],'<=','<-');
//   text[i]:=AnsiReplaceStr(text[i],'<==','<-');

   coloredTextOut(text[i],vars,colors,_canvas,_top_left.Left+(E_PADDING div 2),
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]));
{
   p:=pos('<-',text[i]);
   if(p=0)then
   _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),text[i])
   else
   begin
    _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
                    _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),copy(text[i],1,p-1));
    w:=_canvas.TextWidth(copy(text[i],1,p-1));
    fn:=_canvas.Font.Name;
    _canvas.Font.Name:='Symbol';
    _canvas.TextOut(_top_left.Left+(E_PADDING div 2)+w,
                   _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i])-2,'');
    _canvas.Font.Name:=fn;
    _canvas.TextOut(_top_left.Left+(E_PADDING div 2)+_canvas.TextWidth(copy(text[i],1,p+1)),
                    _top_left.Top+(E_PADDING div 2)+i*_canvas.TextHeight(text[i]),copy(text[i],p+2,length(text[i])));
   end;
}   
 end;

// _canvas.TextOut(_top_left.Left+(E_PADDING div 2),
//                 _top_left.Top+(E_PADDING div 2),text );
end;

function BInstruction.getXml:UTF8string;
begin
 result:='<instruction text="'+xmlTransform(text.commatext)+'" comment="'+xmlTransform(comment.commatext)+'" color="'+Rev(Inttohex(Color,6))+'"></instruction>';
end;

function BInstruction.getPas(_indent:word):TStringList;
var s,sl:TStringList;
    i,j:word;
    ss:UTF8string;
begin
 s:=TStringList.Create;
 if(comment.Count>0) then
 begin
   for i:=0 to comment.Count-1 do
   begin
     s.add(makeIndent(_indent)+'// '+comment[i]+';');
   end;
 end;
 if(text.Count>0) then
 begin
   for i:=0 to text.Count-1 do
   begin
     ss:=text[i];
     ss:=ReplaceRegExpr('[Ll][Ii][Rr][Ee](.*?)',ss,'lire$1',true);
     if(copy(ss,1,4)='lire') then
     begin
       delete(ss,1,4);
       ss:=trim(ss);
       sl:=Explode(',',ss);
       if(sl.count>0) then
         for j:=0 to sl.count-1 do
           if(trim(sl[j])<>'') then s.add(makeIndent(_indent)+'lire '+sl[j]+';');
       sl.free;
     end
     else s.add(makeIndent(_indent)+text[i]+';');
   end;
 end;
 result:=s;
end;

procedure BInstruction.setChildrenColor(_color:TColor);
begin
  color:=_Color;
end;

end.
