unit class_parser;

interface

uses LibXmlParser,
     LibXmlComps,
     Dialogs,
     SysUtils,
     Graphics,
     Forms,
     Contnrs,
     class_root,
     class_element,
     class_alternative,
     class_while,
     class_for,
     class_repeat,
     class_subqueue,
     class_case,
     class_call,
     class_instruction,
     class_jump,
     utils;

type BParser = class
                private
                 XmlParser:TXmlParser;
                 qStack,ifStack,stack,cStack:TObjectStack;
                 root:BRoot;
                 baseq:BSubqueue;
                 lastQ:BSubqueue;
                 lastE:BElement;
                 RootList:TObjectList;
                 multi:boolean;
                 procedure XmlScannerComment(Sender: TObject; Comment: String);
                 procedure XmlScannerContent(Sender: TObject; Content: String);
                 procedure XmlScannerEmptyTag(Sender: TObject; TagName: String; Attributes: TAttrList);
                 procedure XmlScannerEndTag(Sender: TObject; TagName: String);
                 procedure XmlScannerLoadExternal(Sender: TObject; SystemId, PublicId, NotationId: String; var Result: TXmlParser);
                 procedure XmlScannerPI(Sender: TObject; Target, Content: String; Attributes: TAttrList);
                 procedure XmlScannerStartTag(Sender: TObject; TagName: String; Attributes: TAttrList);
                 procedure XmlScannerXmlProlog(Sender: TObject; XmlVersion, Encoding: String; Standalone: Boolean);
                 function XmlScannerTranslateEncoding(Sender: TObject; CurrentEncoding, Source: String): String;

                public
                 constructor create;
                 destructor destroy; override;
                 procedure free;

                 function parse(_filename:String; _root:BRoot): BRoot;
                 function parseInto(_content:String; _q:BSubqueue): BSubqueue;
                 function parseIntoPos(_content:String; _q:BSubqueue; _p:integer): BSubqueue;
                 function parseMDI(_filename:String): TObjectList;
                published

               end;

implementation

constructor BParser.create;
begin
 //XmlParser:=TXmlParser.Create;
 multi:=false;
 lastQ:=nil;
 lastE:=nil;
 //stack:=TObjectStack.Create;
end;

destructor BParser.destroy;
begin
 {if(Assigned(XmlPArser)) then
 begin
  XmlParser.destroy;
  XmlParser:=nil;
 end;}
 //stack.free;
 inherited destroy;
end;

procedure BParser.free;
begin
 {if(Assigned(XmlPArser)) then
 begin
  XmlParser.Free;
  XmlParser:=nil;
 end; }
 //stack.free;
 inherited free;
end;

procedure BParser.XmlScannerComment(Sender: TObject; Comment: String);
begin
end;

procedure BParser.XmlScannerContent(Sender: TObject; Content: String);
begin
end;

procedure BParser.XmlScannerEmptyTag(Sender: TObject; TagName: String; Attributes: TAttrList);
begin

end;

procedure BParser.XmlScannerLoadExternal(Sender: TObject; SystemId, PublicId, NotationId: String; var Result: TXmlParser);
begin

end;

procedure BParser.XmlScannerPI(Sender: TObject; Target, Content: String; Attributes: TAttrList);
begin

end;

procedure BParser.XmlScannerStartTag(Sender: TObject; TagName: String; Attributes: TAttrList);
var el:BElement;
begin
 //showmessage('OPEN :: '+TagName+' --> '+inttostr(stack.Count)+' ==> '+inttostr(qstack.Count));
 if(TagName='root') then
 begin
  if(multi=true) then
  begin
    root:=BRoot.create;
    RootList.Add(root);
  end;
  root.isProgram:=true;
  if(attributes.Value('type')<>'program') then root.isProgram:=false;
  if(attributes.Value('type')='') then root.isProgram:=true;
  root.isNice:=true;
  if(attributes.Value('style')<>'nice') then root.isNice:=false;
  if(attributes.Value('style')='') then root.isNice:=true;
  if(attributes.Value('color')<>'') then root.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  root.text.commatext:=attributes.Value('text');
  root.comment.commatext:=attributes.Value('comment');
  NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  lastE:=root;
  stack.Push(root);
 end
 else if(TagName='instruction') then
 begin
  el:=BInstruction.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='call') then
 begin
  el:=BCall.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='jump') then
 begin
  el:=BJump.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='alternative') then
 begin
  el:=BAlternative.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  (el as BAlternative).qFalse.color:=el.color;
  (el as BAlternative).qTrue.color:=el.color;
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='while') then
 begin
  el:=BWhile.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  (el as BWhile).q.color:=el.color;
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='for') then
 begin
  el:=BFor.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then el.comment.commatext:=attributes.Value('comment');
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  (el as BFor).q.color:=el.color;
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='repeat') then
 begin
  el:=BRepeat.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  (el as BRepeat).q.color:=el.color;
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end
 else if(TagName='case') then
 begin
  el:=BCase.create;
  el.text.commatext:=attributes.Value('text');
  if(attributes.Value('comment')<>'') then
  begin
    el.comment.commatext:=attributes.Value('comment');
    NSDUtils.flagDrawComments:=NSDUtils.flagDrawComments or (trim(attributes.Value('comment'))<>'');
  end;
  if(attributes.Value('color')<>'') then el.color:=StrtoInt('$'+Rev(attributes.Value('color')));
  cStack.Push(el);
  lastE:=el;
  stack.Push(el);
  lastQ.AddElement(el);
 end

 else if(TagName='qCase') then
 begin
  lastQ:=BSubqueue.create;
  ((cStack.Peek) as BCase).qs.Add(lastQ);
  qStack.Push(lastQ);
 end
 else if(TagName='children') then
 begin
  lastQ:=Root.children;
  qStack.Push(lastQ);
 end
 else if(TagName='qTrue') then
 begin
  lastQ:=(lastE as BAlternative).qTrue;
  ifStack.Push((lastE as BAlternative).qFalse);
  qStack.Push(lastQ);
 end
 else if(TagName='qFalse') then
 begin
  lastQ:=(ifStack.pop as BSubqueue);
  qStack.Push(lastQ);
 end
 else if(TagName='qFor') then
 begin
  lastQ:=(lastE as BFor).q;
  qStack.Push(lastQ);
 end
 else if(TagName='qWhile') then
 begin
  lastQ:=(lastE as BWhile).q;
  qStack.Push(lastQ);
 end
 else if(TagName='qRepeat') then
 begin
  lastQ:=(lastE as BRepeat).q;
  qStack.Push(lastQ);
 end
 ;
 //showmessage(TagName+' ==> END');
end;

procedure BParser.XmlScannerEndTag(Sender: TObject; TagName: String);
begin
 //showmessage('CLOSE :: '+TagName+' --> '+inttostr(stack.Count)+' ==> '+inttostr(qstack.Count));
 //showmessage(lastE.ClassName+' --> '+inttostr(stack.Count));
 if(TagName='root') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='instruction') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='call') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='alternative') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='while') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='for') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='repeat') then
 begin
  lastE:=(stack.Pop as BElement);
 end
 else if(TagName='case') then
 begin
  lastE:=(stack.Pop as BElement);
  cStack.Pop;
 end

 else if(TagName='qCase') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  lastQ:=(qStack.Peek as BSubqueue);
  //while(not(lastE is BCase)) do lastE:=(stack.Pop as BElement);
 end
 else if(TagName='children') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  //lastQ:=(qStack.Peek as BSubqueue);
 end
 else if(TagName='qTrue') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  lastQ:=(qStack.Peek as BSubqueue);
 end
 else if(TagName='qFalse') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  lastQ:=(qStack.Peek as BSubqueue);
 end
 else if(TagName='qFor') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  lastQ:=(qStack.Peek as BSubqueue);
 end
 else if(TagName='qWhile') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  lastQ:=(qStack.Peek as BSubqueue);
 end
 else if(TagName='qRepeat') then
 begin
  lastQ:=(qStack.pop as BSubqueue);
  lastQ:=(qStack.Peek as BSubqueue);
 end
 ;
end;

function BParser.XmlScannerTranslateEncoding(Sender: TObject; CurrentEncoding, Source: String): String;
begin

end;

function BParser.parse(_filename:String; _root:BRoot): BRoot;
//var i:word;
begin
 root:=_root;
 XmlParser:=TXmlParser.Create;
 stack:=TObjectStack.Create;
 ifStack:=TObjectStack.Create;
 qStack:=TObjectStack.create;
 cStack:=TObjectStack.Create;
 XmlParser.LoadFromFile(_filename);
 XmlParser.Normalize:=false;
 XmlParser.StartScan;
 XmlParser.Normalize:=false;
 //i:=0;
 While(XmlParser.Scan) do
 begin
  //showmessage('I = '+inttostr(i)+'  --> '+XmlParser.CurName);
  case XmlParser.CurPartType of
      ptStartTag  : XmlScannerStartTag(nil,XmlParser.CurName,XmlParser.CurAttr);
//      ptEmptyTag  : begin
//                     XmlScannerStartTag(nil,XmlParser.CurName,XmlParser.CurAttr);
//                     XmlScannerEndTag(nil,XmlParser.CurName);
//                    end;
//      ptContent,
//      ptCData     : XmlScannerContent(nil,XmlParser.CurContent);
      ptEndTag    : XmlScannerEndTag(nil,XmlParser.CurName);
//      ptComment   : XmlScannerComment(nil,trim(Copy(XmlParser.CurStart, 5, length(XmlParser.CurStart)-length(XmlParser.CurFinal)-6)));
//      ptComment   : XmlScannerComment(nil,XmlParser.CurFinal);
  end;
  //i:=i+1;
 end;
 parse:=root;
 root:=nil;
 stack.free;
 ifStack.Free;
 qStack.free;
 cStack.Free;
 XmlParser.free;
end;

function BParser.parseInto(_content:String; _q:BSubqueue): BSubqueue;                   // V.123
begin
 XmlParser:=TXmlParser.Create;
 stack:=TObjectStack.Create;
 ifStack:=TObjectStack.Create;
 qStack:=TObjectStack.create;
 cStack:=TObjectStack.Create;

 baseq:=_q;
 lastE:=_q.parent;
 stack.Push(lastE);
 lastQ:=_q;
 qStack.Push(_q);

 XmlParser.LoadFromBuffer(PChar(_content));
 XmlParser.Normalize:=false;
 XmlParser.StartScan;
 XmlParser.Normalize:=false;
 While(XmlParser.Scan) do
 begin
  case XmlParser.CurPartType of
      ptStartTag  : XmlScannerStartTag(nil,XmlParser.CurName,XmlParser.CurAttr);
      ptEndTag    : XmlScannerEndTag(nil,XmlParser.CurName);
  end;
 end;
 parseInto:=baseq;
 baseq:=nil;
 stack.free;
 ifStack.Free;
 qStack.free;
 cStack.Free;
 XmlParser.free;
end;

function BParser.parseIntoPos(_content:String; _q:BSubqueue; _p:integer): BSubqueue;
var myQ:BSubqueue;
begin
 XmlParser:=TXmlParser.Create;
 stack:=TObjectStack.Create;
 ifStack:=TObjectStack.Create;
 qStack:=TObjectStack.create;
 cStack:=TObjectStack.Create;

 myQ:=BSubqueue.create;
 baseq:=myQ;
 lastE:=_q.parent;
 stack.Push(lastE);
 lastQ:=myQ;
 qStack.Push(myQ);

 XmlParser.LoadFromBuffer(PChar(_content));
 XmlParser.Normalize:=false;
 XmlParser.StartScan;
 XmlParser.Normalize:=false;
 While(XmlParser.Scan) do
 begin
  case XmlParser.CurPartType of
      ptStartTag  : XmlScannerStartTag(nil,XmlParser.CurName,XmlParser.CurAttr);
      ptEndTag    : XmlScannerEndTag(nil,XmlParser.CurName);
  end;
 end;

 while(myQ.children.Count>0) do
 begin
   _q.children.Insert(_p+1,myQ.children[0]);
   myQ.children.Delete(0);
 end;
 myQ.Free;

 parseIntoPos:=baseq;
 baseq:=nil;
 stack.free;
 ifStack.Free;
 qStack.free;
 cStack.Free;
 XmlParser.free;
end;


procedure BParser.XmlScannerXmlProlog(Sender: TObject; XmlVersion, Encoding: String; Standalone: Boolean);
begin

end;

function BParser.parseMDI(_filename:String): TObjectList;
begin
 multi:=true;
 RootList:=TObjectList.create(false);
 XmlParser:=TXmlParser.Create;
 stack:=TObjectStack.Create;
 ifStack:=TObjectStack.Create;
 qStack:=TObjectStack.create;
 cStack:=TObjectStack.Create;
 XmlParser.LoadFromFile(_filename);
 XmlParser.Normalize:=false;
 XmlParser.StartScan;
 XmlParser.Normalize:=false;
 //i:=0;
 While(XmlParser.Scan) do
 begin
  //showmessage('I = '+inttostr(i)+'  --> '+XmlParser.CurName);
  case XmlParser.CurPartType of
      ptStartTag  : XmlScannerStartTag(nil,XmlParser.CurName,XmlParser.CurAttr);
//      ptEmptyTag  : begin
//                     XmlScannerStartTag(nil,XmlParser.CurName,XmlParser.CurAttr);
//                     XmlScannerEndTag(nil,XmlParser.CurName);
//                    end;
//      ptContent,
//      ptCData     : XmlScannerContent(nil,XmlParser.CurContent);
      ptEndTag    : XmlScannerEndTag(nil,XmlParser.CurName);
//      ptComment   : XmlScannerComment(nil,trim(Copy(XmlParser.CurStart, 5, length(XmlParser.CurStart)-length(XmlParser.CurFinal)-6)));
//      ptComment   : XmlScannerComment(nil,XmlParser.CurFinal);
  end;
  //i:=i+1;
 end;
 parseMDI:=RootList;
 stack.free;
 ifStack.Free;
 qStack.free;
 cStack.Free;
 XmlParser.free;
 multi:=false;
end;


end.
