unit class_root;

interface

uses Graphics,
     Types,
     ContNrs,
     Math,
     Dialogs,
     SysUtils,
     ExtCtrls,
     class_element,
     class_instruction,
     class_alternative,
     class_subqueue,
     class_case,
     class_jump,
     class_call,
     class_for,
     class_while,
     class_repeat,
     StrUtils,
     StdCtrls,
     Classes,
     utils;

type BRoot =
                class(BElement)
                   children:BSubqueue;
                   function PrepareDraw(_canvas:TCanvas): TRect; override; // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect);  overload; override; // passes back the dimensions of the square
                 public
                   selectedElement:Belement;
                   changed:boolean;
                   isProgram:boolean;
                   isNice:boolean;

                   top,left:integer;

                   constructor create; override;
                   destructor destroy; override;
                   procedure Free; override;

                   function Draw(_canvas:TCanvas):tRect; overload;
                   procedure Draw(_image:Timage); overload;

                   function SelectByCoord(_x,_y:word):boolean; override;
                   function SelectElementByCoord(_x,_y:word):BElement; override;
                   function GetByCoord(_x,_y:word):BElement;

                   procedure removeElement(var _element:BElement);
                   procedure removeSelectedElement;

                   procedure addAfterSelected(_element:Belement);
                   procedure addBeforeSelected(_element:Belement);
                   procedure updateSelected(_text,_comment:UTF8string);
                   procedure insertAfterElement(var _insertElement,_afterElement:BElement);
                   procedure moveDownSelected;
                   procedure moveUpSelected;
                   function checkChild(_child,_parent:BElement): boolean;
                   function getACopyOf:BRoot;

                   function getXml:UTF8string; override;
                   function getPas(_indent:word):TStringList; override;
                   procedure clear;

                   procedure setChildrenColor(_color:TColor); override;

                   procedure convert(_el:BElement; _class:UTF8string);
                 private
                 protected
                end;

implementation

constructor BRoot.create;
begin
 inherited create;
 E_DRAWCOL:=clYellow;
 children:=BSubqueue.create;
 children.parent:=self;
 selectedElement:=nil;
 changed:=false;
 isProgram:=true;
 isNice:=true;
end;

destructor BRoot.destroy;
begin
 selectedElement:=nil;
 children.free;
 inherited destroy;
end;

procedure BRoot.Free;
begin
 children.free;
 inherited free;
end;

function BRoot.getACopyOf:BRoot;
var ele:BRoot;
    i:longint;
begin
  ele:=BRoot.create;
  ele.text.text:=self.text.text;
  ele.comment.text:=self.comment.text;
  ele.color:=self.color;
  ele.isProgram:=self.isProgram;
  ele.isNice:=self.isNice;
  ele.children:=(self.children.getACopyOf as BSubqueue);
  ele.selectedElement:=self.selectedElement;  // 1.26
  result:=ele;
end;

function BRoot.PrepareDraw(_canvas:TCanvas): TRect;
var subrect:TRect;
    i:word;
begin
// _canvas.Font.Name:='Comic Sans MS';

 rect.Top:=0;
 rect.Left:=0;
 _canvas.Font.Style:=[fsBold];

 if(isNice=true) then
 begin
   rect.Right:=2*(E_PADDING );
   for i:=0 to text.count-1 do
    if (rect.Right<_canvas.TextWidth(text[i])+2*(E_PADDING ))
     then rect.Right:=_canvas.TextWidth(text[i])+2*(E_PADDING );
   rect.Bottom:=3*E_PADDING+text.count*_canvas.TextHeight('O');
 end
 else
 begin
   rect.Right:=2*(E_PADDING );
   for i:=0 to text.count-1 do
    if (rect.Right<_canvas.TextWidth(text[i])+2*(E_PADDING div 2))
     then rect.Right:=_canvas.TextWidth(text[i])+2*(E_PADDING div 2);
   rect.Bottom:=2*(E_PADDING div 2)+text.count*_canvas.TextHeight('O');
 end;

 _canvas.Font.Style:=[];

 subrect:=children.PrepareDraw(_canvas);
 if(isNice=true) then
 begin
   rect.right:=max(rect.Right,subrect.Right+2*E_PADDING);
 end
 else
 begin
   rect.right:=max(rect.Right,subrect.Right);
 end;
 rect.bottom:=rect.bottom+subrect.bottom;
 result:=rect;
end;

procedure BRoot.Draw(_canvas:TCanvas; _top_left:TRect);
var myrect:TRect;
    drawColor,colcol:TColor;
    i:word;
begin
 if (self.text.Text='') then self.text.Text:='???';
 rect:=_top_left;
 drawColor:=clWhite;
 drawColor:=color;
 if (selected=true) then drawColor:=E_DRAWCOL;

 myrect:=_top_left;
 myrect.Left:=myrect.Left+1;
 myrect.Top:=myrect.Top+1;
 myrect.bottom:=myrect.bottom-1;
 myrect.right:=myrect.right-1;
// _canvas.pen.Color:=clBtnFace;
// _canvas.brush.Color:=clBtnFace;
// _canvas.Rectangle(rect);
 _canvas.pen.Color:=clWhite;
 _canvas.brush.Color:=clWhite;
 _canvas.Rectangle(rect);
// _canvas.FillRect(myrect);

 _canvas.pen.Color:=clBlack;
 _canvas.brush.Color:=drawColor;
 if(isProgram=true) then  _canvas.Rectangle(rect)
 else if(isNice=false) then
 begin
  _canvas.pen.Color:=clNone;
  _canvas.Rectangle(rect)
 end
 else _canvas.RoundRect(rect.Left,rect.Top,rect.Right-1,rect.Bottom-1,30,30); //(rect.right-rect.left) div 3,(rect.Bottom-rect.Top) div 3);
// _canvas.FrameRect(rect);
 if((trim(comment.Text)<>'')and(NSDUtils.flagDrawComments=true)) then
 begin
  colCol:=_canvas.Pen.Color;
  _canvas.Pen.Color:=commentColor;
  _canvas.Pen.Width:=2;
  _canvas.MoveTo(rect.Left+3,rect.Top+2);
   if(isNice=true) then
   begin
     _canvas.LineTo(rect.Left+3,rect.Bottom-3);
   end
   else
   begin
     _canvas.LineTo(rect.Left+3,_canvas.TextHeight('O')*text.count+2*(E_PADDING div 2)-2);
   end;
  _canvas.Pen.Width:=1;
  _canvas.Pen.Color:=colCol;
 end;

 _canvas.Font.Style:=[fsBold];
 if(isNice=true) then
 begin
   for i:=0 to text.Count-1 do
   _canvas.TextOut(rect.left+E_PADDING,
                     rect.top+i*_canvas.TextHeight(text[i])+E_PADDING,text[i]);
 end
 else
 begin
   for i:=0 to text.Count-1 do
   _canvas.TextOut(rect.left+(E_PADDING div 2),
                     rect.top+i*_canvas.TextHeight(text[i])+(E_PADDING div 2),text[i]);
 end;
// _canvas.TextRect(rect,E_PADDING,E_PADDING,text.text);

 _canvas.Font.Style:=[];

 if(isNice=true) then
 begin
   rect.Top:=_canvas.TextHeight('O')*text.count+2*E_PADDING;
   rect.Bottom:=rect.Bottom-E_PADDING;
   rect.Left:=E_PADDING;
   rect.Right:=rect.Right-E_PADDING;
 end
 else
 begin
   rect.Top:=_canvas.TextHeight('O')*text.count+2*(E_PADDING div 2);
   rect.Bottom:=rect.Bottom;
   rect.Left:=0;
   rect.Right:=rect.Right;
 end;
 _canvas.brush.Color:=drawColor;
// _canvas.FrameRect(rect);

 children.Draw(_canvas,rect);

(*
 _canvas.brush.Color:=clNone;
 _canvas.Brush.Style:=bsClear;
 rect:=_top_left;
 if(isNice=false)and(isProgram=false) then
 begin
   _canvas.pen.Color:=clWhite;
   _canvas.Rectangle(rect);
   _canvas.pen.Color:=clBlack;
   _canvas.RoundRect(rect.Left,rect.Top,rect.Right,rect.Bottom,30,30);
 end;
*)
 rect:=_top_left;
end;

function BRoot.Draw(_canvas:TCanvas):trect;
var myRect:Trect;
begin
 myrect:=self.PrepareDraw(_canvas);
 self.Draw(_canvas,myRect);
 result:=myrect;
end;

procedure BRoot.Draw(_image:Timage);
var myRect:Trect;
begin
 myrect:=self.PrepareDraw(_image.canvas);
 _image.Width:=myrect.Right;
 _image.Height:=myRect.Bottom;
 _image.Picture.Bitmap.Width:=myrect.Right;
 _image.Picture.Bitmap.Height:=myrect.Bottom;
 self.Draw(_image.canvas,myRect);
end;


function BRoot.SelectByCoord(_x,_y:word):boolean;
var sel:boolean;
begin
 inherited SelectByCoord(_x,_y);
 sel:=children.SelectByCoord(_x,_y);
 if(sel=true) then selected:=false;
 result:=(selected or sel);
end;

function BRoot.GetByCoord(_x,_y:word):BElement;
var sel:BElement;
begin
 sel:=children.SelectElementByCoord(_x,_y);
 if(sel=nil) then sel:=inherited SelectElementByCoord(_x,_y);
 result:=sel;
end;

function BRoot.SelectElementByCoord(_x,_y:word):BElement;
var sel:BElement;
begin
 selectedElement:=inherited SelectElementByCoord(_x,_y);
 sel:=children.SelectElementByCoord(_x,_y);
 if(sel<>nil) then
 begin
  selectedElement:=sel;
  sel.selected:=true;
 end;
 result:=selectedElement;
end;

procedure BRoot.removeElement(var _element:BElement);
begin
 if (_element<>nil) then
 begin
  _element.selected:=false;
  if (not(_element is BSubqueue))and(not(_element is BRoot)) then
  begin
   (_element.parent as BSubqueue).RemoveElement(_element);
   changed:=true;
  end;
 end;
end;

procedure BRoot.insertAfterElement(var _insertElement,_afterElement:BElement);
var i:word;
begin
 if ((_afterElement<>nil)and(_insertElement<>nil)) then
 begin
  if (not(_afterElement is BSubqueue))and(not(_afterElement is BRoot)) then
  begin
    i:=(_afterElement.parent as BSubqueue).children.IndexOf(_afterElement);
    (_afterElement.parent as BSubqueue).children.Insert(i+1,_insertElement);
    _insertElement.parent:=_afterElement.parent;
    selectedElement.selected:=false;
    SelectByCoord(0,0);
    selectedElement:=_insertElement;
    selectedElement.selected:=true;
    changed:=true;
  end;
 end;
end;

procedure BRoot.removeSelectedElement;
begin
 removeElement(selectedElement);
 selectedElement:=nil;
 changed:=true;
end;

procedure BRoot.addAfterSelected(_element:Belement);
var i:word;
begin
 if (selectedElement<>nil) then
 begin
  if (selectedElement is BSubqueue) then
  begin
   (selectedElement as BSubqueue).AddElement(_element);
    selectedElement.selected:=false;
    SelectByCoord(0,0);
    _element.selected:=true;
    selectedElement:=_element;
    changed:=true;
  end
  else if(selectedElement.parent is BSubqueue) then
       begin
         i:=(selectedElement.parent as BSubqueue).children.IndexOf(selectedElement);
         (selectedElement.parent as BSubqueue).children.Insert(i+1,_element);
         _element.parent:=selectedElement.parent;
         selectedElement.selected:=false;
         SelectByCoord(0,0);
         _element.selected:=true;
         selectedElement:=_element;
         changed:=true;
       end;
 end;
 //else showmessage('No element selected!');
end;

procedure BRoot.addBeforeSelected(_element:Belement);
var i:word;
begin
 if (selectedElement<>nil) then
 begin
  if (selectedElement is BSubqueue) then
  begin
   (selectedElement as BSubqueue).AddElement(_element);
    selectedElement.selected:=false;
    SelectByCoord(0,0);
    _element.selected:=true;
    selectedElement:=_element;
    changed:=true;
  end
  else if(selectedElement.parent is BSubqueue) then
       begin
         i:=(selectedElement.parent as BSubqueue).children.IndexOf(selectedElement);
         (selectedElement.parent as BSubqueue).children.Insert(i,_element);
         _element.parent:=selectedElement.parent;
         selectedElement.selected:=false;
         SelectByCoord(0,0);
         _element.selected:=true;
         selectedElement:=_element;
         changed:=true;
       end;
 end;
end;

procedure BRoot.updateSelected(_text,_comment:UTF8string);
begin
 if (selectedElement<>nil) then
 begin
  if not(selectedElement is BSubqueue) then
  begin
    if(selectedElement is BCase) then  (selectedElement as BCase).setTextLines(_text) // 1.21
    else selectedElement.text.text:=_text;                                            // 1.21
    selectedElement.comment.text:=_comment;
    changed:=true;
  end;
 end;
end;

function BRoot.getXml:UTF8string;
var t,n:UTF8string;
begin
 if(isProgram=true) then t:='program'
 else t:='sub';
 if(isNice=true) then n:='nice'
 else n:='abbr';
 result:='<?xml version="1.0" encoding="ISO-8859-1"?>'+#13+
         '<root text="'+xmlTransform(text.commatext)+'" comment="'+xmlTransform(comment.commatext)+'" type="'+t+'" style="'+n+'">'+#13+
         ' <children>'+#13+
         children.getXml+#13+
         ' </children>'+#13+
         '</root>';
end;

function BRoot.getPas(_indent:word):TStringList;
var s:TStringList;
    i:integer;
begin
 s:=TStringList.Create;
 if(comment.Count>0) then
 begin
   for i:=0 to comment.Count-1 do
   begin
     s.add(makeIndent(_indent)+'// '+comment[i]+';');
   end;
 end;
 if(isProgram=true)
 then s.add(makeIndent(_indent)+'program '+trim(text.text)+';')
 else
 if(((Pos('):' ,trim(text.text))<>0) or
     (Pos(') :',trim(text.text))<>0) or
     ((Pos(':',trim(text.text))<>0) and
      (Pos('(',trim(text.text))=0))) and
    (isProgram=false))
 then s.add(makeIndent(_indent)+'function '+trim(text.text)+';')
 else s.add(makeIndent(_indent)+'procedure '+trim(text.text)+';');
 s.add('');
 s.add(makeIndent(_indent)+'begin');
 s.AddStrings(children.getPas(_indent+E_INDENT));
 if(isProgram=true) then s.add(makeIndent(_indent)+'end.')
 else s.add(makeIndent(_indent)+'end;');
 result:=s;
end;

procedure BRoot.clear;
begin
 children.clear;
 changed:=true;
 isProgram:=true;
 text.Text:='???';
end;

procedure BRoot.moveDownSelected;
var i:word;
begin
 if (selectedElement<>nil) then
 begin
  if(selectedElement.parent is BSubqueue)then
  begin
         i:=(selectedElement.parent as BSubqueue).children.IndexOf(selectedElement);
         if(i+1<(selectedElement.parent as BSubqueue).children.Count)then
         begin
          (selectedElement.parent as BSubqueue).children.Delete(i);
          (selectedElement.parent as BSubqueue).children.Insert(i+1,selectedElement);
         end;
         changed:=true;
  end;
 end;
end;

procedure BRoot.moveUpSelected;
var i:word;
begin
 if (selectedElement<>nil) then
 begin
  if(selectedElement.parent is BSubqueue)then
  begin
         i:=(selectedElement.parent as BSubqueue).children.IndexOf(selectedElement);
         if(i-1>=0)then
         begin
          (selectedElement.parent as BSubqueue).children.Delete(i);
          (selectedElement.parent as BSubqueue).children.Insert(i-1,selectedElement);
         end;
         changed:=true;
  end;
 end;
end;

function BRoot.checkChild(_child,_parent:BElement): boolean;
var tmp:BElement;
    res:boolean;
begin
 tmp:=_child;
 res:=false;
 if(tmp<>nil) then
 while((tmp.parent<>nil)and(res=false))do
 begin
  if(tmp.parent=_parent) then res:=true;
  tmp:=tmp.parent;
 end;
 result:=res;
end;

procedure Broot.setChildrenColor(_color:TColor);
begin
  color:=_color;
  children.setChildrenColor(_color);
end;

procedure Broot.convert(_el:BElement; _class:UTF8string);                           // 1.25
var el:BElement;
begin
  if(_el.ClassNameIs('BInstruction') or
     _el.ClassNameIs('BCall') or
     _el.ClassNameIs('BJump')) then
  begin
    if(_class='BJump') then el:=BJump.create
    else if(_class='BCall') then el:=BCall.create
    else if(_class='BAlternative') then el:=BAlternative.create
    else if(_class='BFor') then el:=BFor.create
    else if(_class='BWhile') then el:=BWhile.create
    else if(_class='BRepeat') then el:=BRepeat.create
    else if(_class='BCase') then el:=BCase.create
    else el:=BInstruction.create;
    el.text.Text:=_el.text.text;
    if(_class='BCase') then
    begin
       el.text.add('<?>');
       el.text.add('else');
      (el as BCase).setTextLines(el.text.text);
    end;
    el.color:=_el.color;
    el.comment.text:=_el.comment.text;
    el.selected:=_el.selected;
    self.insertAfterElement(el,_el);
    self.removeElement(_el);
  end;
end;

end.

