unit class_subqueue;

interface

uses Graphics,
     Types,
     ContNrs,
     Dialogs,
     Math,
     class_element,
     class_instruction,
     StrUtils,
     Classes;

type BSubqueue =
                class(BElement)
                   children:TObjectList;
                   function PrepareDraw(_canvas:TCanvas): TRect;  // passes back the dimensions of the square
                   procedure Draw(_canvas:TCanvas; _top_left:TRect);  // passes back the dimensions of the square
                   function getXml:UTF8string; override;
                   function getPas(_indent:word):TStringList; override;
                 public
                   singleInstruction:boolean;
                   dontCloseRec:boolean;
                   constructor create; overload; virtual;
                   destructor destroy; virtual;
                   procedure Free; override;

                   procedure AddElement(_element:BElement);
                   procedure RemoveElement(_element:BElement); overload;
                   procedure RemoveElement(_index:word); overload;
                   function GetElement(_index:word): BElement;

                   function AddIf(_text:UTF8string): BElement;
                   function AddWhile(_text:UTF8string): BElement;
                   function AddRepeat(_text:UTF8string): BElement;
                   function AddFor(_text:UTF8string): BElement;
                   function AddInstruction(_text:UTF8string): BElement;

                   function SelectByCoord(_x,_y:word):boolean; override;
                   function SelectElementByCoord(_x,_y:word):BElement; override;
                   procedure setChildrenColor(_color:TColor); override;
                   function getACopyOf:BElement; override;

                   procedure clear;
                 private
                 protected
                end;



implementation

uses class_repeat,
     class_for,
     class_while,
     class_alternative;

constructor BSubqueue.create;
begin
 inherited create;
 children:=TObjectList.Create(false);
 singleInstruction:=false;
 dontCloseRec:=false;
end;

destructor BSubqueue.destroy;
var el:Belement;
begin
 while(children.Count<>0) do
 begin
  el:=(children.Items[0] as Belement);
  children.Delete(0);
  el.free;
 end;
 children.free;
 inherited destroy;
end;

procedure BSubqueue.Free;
begin
  children.free;
  inherited free;
end;

function BSubqueue.getACopyOf:BElement;
var ele:BElement;
    i:longint;
begin
  ele:=BSubqueue.create;
  ele.color:=self.color;
  if(self.children.count>0) then
    for i:=0 to self.children.count-1 do
      (ele as BSubqueue).AddElement((self.children[i] as BElement).getACopyOf);
  result:=ele;
end;

function BSubqueue.PrepareDraw(_canvas:TCanvas): TRect;
var subrect:TRect;
    i:word;
    fn:UTF8string;
begin
 rect.Top:=0;
 rect.Left:=0;
 rect.Right:=0;
 rect.Bottom:=0;
 if (children.Count>0) then
 begin
   for i:=0 to children.count-1 do
   begin
      subrect:=(children.items[i] as BElement).PrepareDraw(_canvas);
      // hei ass iergendeppes !!!
      rect.right:=max(rect.Right,subrect.Right);
      rect.bottom:=rect.bottom+subrect.bottom;
   end;
 end
 else
 begin
  // allocate some space for "nothing"
  rect.right:=2*E_PADDING;
  fn:=_canvas.Font.Name;
//  _canvas.Font.Name:='Monotype Sorts';
  rect.bottom:=_canvas.TextHeight('%')+2*(E_PADDING div 2);
  _canvas.Font.Name:=fn;
 end;
 result:=rect;
end;


procedure BSubqueue.Draw(_canvas:TCanvas; _top_left:TRect);
var myrect,subrect:TRect;
    i:word;
    drawColor:TColor;
    fn:UTF8string;
begin
 drawColor:=clWhite;
 drawColor:=color;
 if (selected=true) then drawColor:=E_DRAWCOL;
 rect:=_top_left;

 myrect:=_top_left;
 myrect.Bottom:=0;

 if (children.Count>0) then
 begin
  for i:=0 to children.count-1 do
  begin
   subrect:=(children.items[i] as BElement).PrepareDraw(_canvas);
   myrect.bottom:=myrect.top+subrect.bottom+1;
   if(i=children.Count-1) then myrect.bottom:=_top_left.Bottom;
   (children.items[i] as BElement).draw(_canvas,myrect);
   myrect.bottom:=myrect.bottom-1;
   myrect.Top:=myrect.Top+subrect.Bottom;
  end;
 end
 else
 begin
  // draw "nothing"
  _canvas.brush.Color:=clBlack;
  _canvas.FillRect(_top_left);

  rect:=_top_left;
  _canvas.brush.Color:=drawColor;
  myrect:=_top_left;
  myrect.Left:=myrect.Left+1;
  myrect.Top:=myrect.Top+1;
  myrect.bottom:=myrect.bottom-1;
  myrect.right:=myrect.right-1;
  _canvas.FillRect(myrect);

  _canvas.brush.Color:=drawColor;
  fn:=_canvas.Font.Name;
(*
  _canvas.Font.Name:='Symbol';
  _canvas.TextOut(_top_left.Left+((_top_left.right-_top_left.Left) div 2) - (_canvas.TextWidth(chr(198)) div 2),
                  _top_left.top +((_top_left.Bottom-_top_left.top) div 2) - (_canvas.TextHeight(chr(198))div 2),
                  chr(198));
*)

  _canvas.TextOut(_top_left.Left+((_top_left.right-_top_left.Left) div 2) - (_canvas.TextWidth('%') div 2),
                  _top_left.top +((_top_left.Bottom-_top_left.top) div 2) - (_canvas.TextHeight('%') div 2),
                  '%');
  _canvas.Font.Name:=fn;
 end;

end;

procedure BSubqueue.AddElement(_element:BElement);
begin
 children.Add(_element);
 _element.parent:=self;
end;

procedure BSubqueue.RemoveElement(_element:BElement);
begin
 _element.parent:=nil;
 children.Remove(_element);
end;

procedure BSubqueue.RemoveElement(_index:word);
begin
 (children.Items[_index] as BElement).parent:=nil;
 children.Delete(_index);
end;

function BSubqueue.GetElement(_index:word): BElement;
begin
 result:=(children.Items[_index] as BElement);
end;

function BSubqueue.SelectByCoord(_x,_y:word):boolean;
var i:word;
    sel,res:boolean;
begin
 inherited SelectByCoord(_x,_y);
 res:=false;
 if (children.Count>0) then
 begin
  for i:=0 to children.count-1 do
  begin
   sel:=(children.items[i] as BElement).SelectByCoord(_x,_y);
   if (sel=true) then res:=true;
  end;
  selected:=false;
 end;
 result:=(selected or res);
end;

function BSubqueue.SelectElementByCoord(_x,_y:word):BElement;
var i:word;
    sel,res:BElement;
begin
 result:=inherited SelectElementByCoord(_x,_y);
 res:=nil;
 if (children.Count>0) then
 begin
  for i:=0 to children.count-1 do
  begin
     sel:=(children.items[i] as BElement).SelectElementByCoord(_x,_y);
     if (sel<>nil) then res:=sel;
  end;
  if(res<>nil) then result:=res;
 end;
end;

function BSubqueue.getXml:UTF8string;
var i:word;
    s:UTF8string;
begin
 s:='';
 if (children.Count>0) then
  for i:=0 to children.count-1 do s:=s+#13+(children.items[i] as BElement).getXml;
 result:=s;
end;

function BSubqueue.getPas(_indent:word):TStringList;
var i:word;
    s:TStringList;
begin
 s:=TStringList.create;
 if (children.Count>0) then
  for i:=0 to children.count-1 do s.addStrings((children.items[i] as BElement).getPas(_indent));
 result:=s;
end;

procedure BSubqueue.clear;
begin
 while(children.Count>0) do children.Delete(0);
end;

function BSubqueue.AddIf(_text:UTF8string): BElement;
var ele:BAlternative;
begin
 ele:=BAlternative.create;
 ele.text.Text:=_text;
 self.AddElement(ele);
 result:=ele;
end;

function BSubqueue.AddWhile(_text:UTF8string): BElement;
var ele:BWhile;
begin
 ele:=BWhile.create;
 ele.text.Text:=_text;
 self.AddElement(ele);
 result:=ele;
end;

function BSubqueue.AddRepeat(_text:UTF8string): BElement;
var ele:BRepeat;
begin
 ele:=BRepeat.create;
 ele.text.Text:=_text;
 self.AddElement(ele);
 result:=ele;
end;

function BSubqueue.AddFor(_text:UTF8string): BElement;
var ele:BFor;
begin
 ele:=BFor.create;
 ele.text.Text:=_text;
 self.AddElement(ele);
 result:=ele;
end;

function BSubqueue.AddInstruction(_text:UTF8string): BElement;
var ele:BInstruction;
begin
 ele:=BInstruction.create;
 ele.text.Text:=_text;
 self.AddElement(ele);
 result:=ele;
end;

procedure BSubqueue.setChildrenColor(_color:TColor);
var i:word;
begin
 color:=_Color;
 if(children.Count>0) then
   for i:=0 to children.Count-1 do
     (children[i] as BElement).setChildrenColor(_color);
end;


end.
